package anywheresoftware.b4a.objects;

import java.util.HashMap;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;
import android.os.Build.VERSION;
import android.service.notification.NotificationListenerService;
import android.service.notification.StatusBarNotification;
import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

@Hide
public class NotificationListenerWrapper extends NotificationListenerService{
	public NotificationListener nl;
	@Override
	public void onCreate() {
		startService(new Intent(this, this.getClass())); //Important if the service will be bound instead of started.
    }
	@Override
	public void onNotificationPosted(StatusBarNotification arg0) {
		BA.Log("onNotificationPosted");
		if (nl != null) {
			nl.ba.raiseEvent(this, nl.eventName + "_notificationposted", AbsObjectWrapper.ConvertToWrapper(
					new StatusBarNotificationWrapper(), arg0));
		}
	}
	
	

	@Override
	public void onNotificationRemoved(StatusBarNotification arg0) {
		
		if (nl != null) {
			nl.ba.raiseEvent(this, nl.eventName + "_notificationremoved", AbsObjectWrapper.ConvertToWrapper(
				new StatusBarNotificationWrapper(), arg0));
		}
	}
	/**
	 * NotificationListener allows you to access the device notifications.
	 *This is only supported by Android 4.3+.
	 *See the tutorial in the forum for more information.
	 */
	@Events(values={"NotificationPosted (SBN As StatusBarNotification)",
			"NotificationRemoved (SBN As StatusBarNotification)"})
	@Version(2.00f)
	@ShortName("NotificationListener")
	public static class NotificationListener {
		BA ba;
		String eventName;
		private NotificationListenerWrapper owner;
		/**
		 * Initializes the object and sets the subs that will handle the events.
		 */
		public void Initialize(BA ba, String EventName) {
			this.ba = ba;
			this.eventName = EventName.toLowerCase(BA.cul);
			owner = (NotificationListenerWrapper) ba.service;//<---- reference to the service
			owner.nl = this;
		}
		
		/**
		 * Clears all non-ongoing notifications.
		 */
		public void ClearAll() {
			owner.cancelAllNotifications();
		}
		/**
		 * Clears the given notification (if it is not an ongoing notification).
		 */
		public void ClearNotification(StatusBarNotificationWrapper SBN) {
			StatusBarNotification sbn = SBN.getObject();
			if (VERSION.SDK_INT >= 21) {
				try {
					String key = (String) sbn.getClass().getMethod("getKey").invoke(sbn);
					owner.getClass().getMethod("cancelNotification", String.class)
						.invoke(this, key);
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
			else
				owner.cancelNotification(sbn.getPackageName(), sbn.getTag(), sbn.getId());
		}
		/**
		 * Causes the listener to repost all the active notifications.
		 */
		public void GetActiveNotifications() {
			StatusBarNotification[] all = owner.getActiveNotifications();
			if (all != null) {
				for (StatusBarNotification sbn : all) {
					ba.raiseEvent(this, eventName + "_notificationposted", AbsObjectWrapper.ConvertToWrapper(
							new StatusBarNotificationWrapper(), sbn));
				}
			}
		}
	}
	
	@ShortName("StatusBarNotification")
	public static class StatusBarNotificationWrapper extends AbsObjectWrapper<StatusBarNotification> {
		/**
		 * Returns the notification package name.
		 */
		public String getPackageName() {
			return getObject().getPackageName();
		}
		/**
		 * Returns the notification id.
		 */
		public int getId() {
			return getObject().getId();
		}
		/**
		 * Returns the notification ticker text field.
		 */
		public String getTickerText() {
			CharSequence cs = getObject().getNotification().tickerText;
			return cs == null ? "" : cs.toString();
		}
		/**
		 * Returns the internal notification object.
		 */
		public NotificationWrapper getNotification() {
			NotificationWrapper nw = new NotificationWrapper();
			nw.setObject(getObject().getNotification());
			return nw;
		}
	}

}
